﻿; ListWindow.ahk[UTF-8] (2025/02/07 08:00)
; for AutoHotKey v2.0.19 or later
;
; 各種ブラウザなどのウィンドウ一覧を取得してテキストに吐き出す
; 全ウィンドウと、空行を挟んで通常メインウィンドウにされていると思われるウィンドウ
; (WS_VISIBLEがONでWS_DISABLEDがOFFであるウィンドウ)を列挙
;

#WinActivateForce
/* 非表示のウィンドウも捜索対象としておく */
DetectHiddenWindows True

;//MouseGetPosはスクリーンを対象とする
CoordMode("Mouse", "Screen")

;//ToolTipはスクリーンを対象とする
CoordMode("ToolTip", "Screen")

WM_SYSCOMMAND := 0x112
SC_MINIMIZE := 0xF020
SC_MAXIMIZE := 0xF030
SC_RESTORE := 0xF120
WS_DISABLED := 0x8000000
WS_VISIBLE := 0x10000000

IniFile    := "ListWindow.ini"
IniSect    := "APP"
OutputFile := "ListWindow.txt"

;iniファイルから件数を読み込む
count := IniRead(IniFile, IniSect, "MAX", 0)
if( count = 0 ){
	MsgBox("iniファイルに定義がありません")
	ExitApp()
}
countM := count

showToolTip := Number(IniRead(IniFile, IniSect, "ToolTip", "0"))

if( showToolTip ){
	hwndToolTips := ToolTip( "ウィンドウ情報を収集しています", A_ScreenWidth / 2, A_ScreenHeight / 2, 1 )
	WinGetPos( &OutX, &OutY, &OutW, &OutH, "ahk_id " hwndToolTips )
	WinMove( (A_ScreenWidth - OutW) / 2, (A_ScreenHeight - OutH) / 2, OutW, OutH, "ahk_id " hwndToolTips )
}

/* クラス名文字列の長さ、出力可否 */
ClassLen := Number(IniRead(IniFile, IniSect, "ClassLen", "50"))
clen := ClassLen
SPC_C := " "
while( clen > 0 ){
	SPC_C := SPC_C " "
	clen := clen - 1
}
ClassOutput := Number(IniRead(IniFile, IniSect, "ClassOutput", "0"))
if( ClassOutput = 0 ){
	ClassLen := 0
	SPC_C := " "
}

/* タイトル文字列の長さ、出力可否 */
TitleLen := Number(IniRead(IniFile, IniSect, "TitleLen", "50"))
tlen := TitleLen
SPC_T := " "
while( tlen > 0 ){
	SPC_T := SPC_T " "
	tlen := tlen - 1
}
TitleOutput := Number(IniRead(IniFile, IniSect, "TitleOutput", "0"))
if( TitleOutput = 0 ){
	TitleLen := 0
	SPC_T := " "
}

TextLen := ClassLen + 42 ;//class名までの固定長部分の長さを足す

/* コマンドライン文字列の長さ、出力可否 */
ComLen := Number(IniRead(IniFile, IniSect, "ComLen", "512"))
mlen := ComLen
SPC_M := " "
while( mlen > 0 ){
	SPC_M := SPC_M " "
	mlen := mlen - 1
}
ComOutput := Number(IniRead(IniFile, IniSect, "ComOutput", "0"))
if( ComOutput = 0 ){
	ComLen := 0
	SPC_M := " "
}


listText := Array()
listTextAll := Array()
idxWin := 0

idx := 1
while( count > 0 ){
	;フルパスの読み込み(PATH1～)
	strTemp := IniRead(IniFile, IniSect, "PATH" String(idx), "")

	if( showToolTip ){
		hwndToolTips := ToolTip( "ウィンドウを列挙しています(" idx "/" countM ")`r`n" strTemp, A_ScreenWidth / 2, A_ScreenHeight / 2, 1 )
		WinGetPos( &OutX, &OutY, &OutW, &OutH, "ahk_id " hwndToolTips )
		WinMove( (A_ScreenWidth - OutW) / 2, (A_ScreenHeight - OutH) / 2, OutW, OutH, "ahk_id " hwndToolTips )
	}

	getWindowList(strTemp)
	count := count - 1
	idx := idx + 1
}

msgText := ""
for tmpText in listTextAll{
	msgText := msgText tmpText
}
msgText := msgText "`r`n"
for tmpText in listText{
	msgText := msgText tmpText
}


/* 新規ファイルに出力するモードの場合は削除 */
if( Number(IniRead(IniFile, IniSect, "NewFile", "0")) ){
	try{
		FileDelete(OutputFile)
	}
}

;A_Clipboard := msgText
FileAppend("`r`n"  msgText "`r`n", OutputFile)

/* 出力ファイルをすぐに開く */
if( Number(IniRead(IniFile, IniSect, "OpenFile", "0")) ){
	try{
		Run(OutputFile)
	}
}


ExitApp()



getWindowList(param){
	global idxWin, listText, listTextAll, SPC
	listWin := WinGetList("ahk_exe " param)
	for hWin in listWin{
		idxWin++
		style := "0x00000000"
		styleEx := "0x00000000"
		class := ""
		title := ""
		path := ""
		comLine := ""
		strIdx := "0000" String(idxWin)
		strIdx := SubStr(strIdx, -4)

		try{
			style := WinGetStyle("ahk_id " hWin)
			styleEx := WinGetExStyle("ahk_id " hWin)
			class := WinGetClass("ahk_id " hWin)
			title := WinGetTitle("ahk_id " hWin)
			path := WinGetProcessPath("ahk_id " hWin)
			strStyle := "0x" SubStr("00000000" StrUpper(Format("{1:X}",style)),-8)
			strStyleEx := "0x" SubStr("00000000" StrUpper(Format("{1:X}",styleEx)),-8)

			if( ComOutput ){
				try{
					;//起動オプションを取得
					pid := WinGetPID("ahk_id " hWin)
					;//WMIサービスオブジェクトを取得
					wmi := ComObjGet("winmgmts:")
					;//クエリーを実行し、一致するプロセスを取得
					queryEnum := wmi.ExecQuery(""
								. "Select * from Win32_Process where ProcessId=" . pid)
								._NewEnum()
					;//最初のマッチング処理を取得
					if queryEnum(&proc){
						comLine := proc.CommandLine
					}
					wmi := ""
					queryEnum := ""
				}
			}

		} catch Error as err {
			path := ""
		}

		try{
			WinGetPos( &OutX, &OutY, &OutW, &OutH, "ahk_id " hWin )
		} catch Error as err {
			hWin := 0
			OutX := 0
			OutY := 0
			OutW := 0
			OutH := 0
		}
		OutX := SubStr("      " OutX, -6)
		OutY := SubStr("      " OutY, -6)
		OutW := SubStr("      " OutW, -6)
		OutH := SubStr("      " OutH, -6)

		;//出力テキスト整形
		text := "[" strIdx "] Style=" strStyle ",Ex=" strStyleEx ",PosSize(" OutX "," OutY ") [" OutW "x" OutH "]"

		;//クラス名を出力する場合
		if( ClassOutput ){
			class := ",class=" SubStrA(class SPC_C, 1, ClassLen)
		}else{
			class := ""
		}

		;//ウィンドウタイトルを出力する場合
		if( TitleOutput ){
			/* SubStrA 関数テスト */
			;title := ",title=" title "|" ;切り取らない場合
			;title := ",title=" SubStrA(title, 7, 10) SPC_T ;始点と長さを指定
			;title := ",title=" SubStrA(title, 7) SPC_T ;長さを省略
			;title := ",title=" SubStrA(title, -7) SPC_T ;始点を負数にして逆から切り取り
			;title := ",title=" SubStrA(title, 3, -9) "|" ;長さを負数にして後ろから削除
			;title := ",title=" SubStrA(title, 3, 11) SPC_T ;終端が全角で分断されて ? になる
			;title := ",title=" SubStrA(title, 2, 11) SPC_T ;先頭が全角で分断されて ? になる
			;title := ",title=" SubStrA(title, 2, 10) SPC_T ;先頭と終端が全角で両端分断され ? になる
			;title := ",title=" SubStrA(title, 0, 11) SPC_T ;位置ゼロは空文字を返す
			/* SubStrA 関数テスト */

			title := ",title=" SubStrA(title SPC_T, 1, TitleLen)
		}else{
			title := ""
		}

		;//コマンドラインオプションを出力する場合
		if( ComOutput ){
			path := SubStrA(path " ( " comLine " )" SPC_M, 1, ComLen)
		}

		if( path != "" ){
			text := text class title "," path "`r`n"
			if( (style & WS_VISIBLE) and not(style & WS_DISABLED ) ){
				;//メインウィンドウと思われるもの
				listText.push(text)
			}else{
				;//その他のウィンドウ
				listTextAll.push(text)
			}
		}
	}
}

/* Unicodeとして文字数が扱われるのでAscii換算 */
StrLenA(str){
	Return StrPut(str, "cp0") - 1
}

/* Unicodeとして文字数が扱われるのでAscii換算 */
SubStrA(str,pos,len:=0){
	out := ""
	tmp := ""
	idx := 0
	count := 0

	;//開始位置が0や長さ以上の場合は空文字を返す
	if( pos = 0 or pos > StrLenA(str) ){
		Return out
	}
	;//開始位置が負数で左端を越える場合は先頭から全て
	if( -1 * pos > StrLenA(str) ){
		pos := 1
		len := 0
	}
	;//長さが負数の場合はその文字数末尾から削る
	if( len < 0 ){
		len := StrLenA(str)+len-pos+1
	}
	;//長さが0の時は開始位置以降を返す
	if( len = 0 ){
		len := StrLenA(str)
	}
	
	if( pos > 0 ){
		;//順方向
		While( idx < pos ){
			tmp := SubStr(str, idx, 1)
			if( StrLenA(tmp) = 2 ){
				pos--
				if( idx = pos ){
					out := "?" out
					len--
					pos++
				}
			}
			idx++
		}
		idx := 0
		While( count < len ){
			tmp := SubStr(str, pos + idx, 1)
			if( StrLenA(tmp) = 2 ){
				count++
				;//全角文字が足される事で長さを超える時
				if( count = len ){
					out := out "?"
				}else{
					out := out tmp
				}
			}else{
				out := out tmp
			}
			idx++
			count++
		}
	}else{
		;//逆方向の時は位置が負数指定されているので絶対値ぶん末尾から取得
		len := -1 * pos
		While( count < len ){
			tmp := SubStr(str, -1)
			str := SubStr(str, 1, StrLen(str)-1)
			if( StrLenA(tmp) = 2 ){
				count++
				;//全角文字が足される事で長さを超える時
				if( count = len ){
					out := "?" out
				}else{
					out := tmp out
				}
			}else{
				out := tmp out
			}
			count++
		}
	}
	Return out
}

;
; クリップボードにコピー
;   A_Clipboard := ""
;
; クリップボードをコンソールに出力
;   powershell -command "get-clipboard"
;

